@@ -2,7 +2,6 @@
 # Copyright (c) 2014 Navneet Singh.  All rights reserved.
 # Copyright (c) 2014 Glenn Gobeli.  All rights reserved.
 # Copyright (c) 2014 Clinton Knight.  All rights reserved.
-# Copyright (c) 2015 Alex Meade.  All rights reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -15,45 +14,25 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-"""NetApp API for Data ONTAP and OnCommand DFM.
+"""
+NetApp API for Data ONTAP and OnCommand DFM.
 
 Contains classes required to issue API calls to Data ONTAP and OnCommand DFM.
 """
-import random
-import urllib
 
-from eventlet import greenthread
-from eventlet import semaphore
+import copy
+import urllib2
+
 from lxml import etree
 from oslo_log import log as logging
-from oslo_serialization import jsonutils
-from oslo_utils import netutils
-import requests
-from requests.adapters import HTTPAdapter
-from requests import auth
-from requests.packages.urllib3.util.retry import Retry
+import six
 
 from cinder import exception
 from cinder.i18n import _
-from cinder import ssh_utils
-from cinder.volume.drivers.netapp import utils as na_utils
-from cinder.volume import volume_utils
 
 LOG = logging.getLogger(__name__)
 
-# ZAPI API error codes.
-EAPIERROR = '13001'
-EAPIPRIVILEGE = '13003'
-EAPINOTFOUND = '13005'
-ESNAPSHOTNOTALLOWED = '13023'
 ESIS_CLONE_NOT_LICENSED = '14956'
-EOBJECTNOTFOUND = '15661'
-ESOURCE_IS_DIFFERENT = '17105'
-ERELATION_EXISTS = '17122'
-ERELATION_NOT_QUIESCED = '17127'
-ENOTRANSFER_IN_PROGRESS = '17130'
-EANOTHER_OP_ACTIVE = '17131'
-ETRANSFER_IN_PROGRESS = '17137'
 
 
 class NaServer(object):
@@ -72,7 +51,7 @@ class NaServer(object):
     def __init__(self, host, server_type=SERVER_TYPE_FILER,
                  transport_type=TRANSPORT_TYPE_HTTP,
                  style=STYLE_LOGIN_PASSWORD, username=None,
-                 password=None, port=None, api_trace_pattern=None):
+                 password=None, port=None):
         self._host = host
         self.set_server_type(server_type)
         self.set_transport_type(transport_type)
@@ -83,18 +62,17 @@ class NaServer(object):
         self._password = password
         self._refresh_conn = True
 
-        if api_trace_pattern is not None:
-            na_utils.setup_api_trace_pattern(api_trace_pattern)
+        LOG.debug('Using NetApp controller: %s' % self._host)
 
-        LOG.debug('Using NetApp controller: %s', self._host)
+    def get_transport_type(self):
+        """Get the transport type protocol."""
+        return self._protocol
 
     def set_transport_type(self, transport_type):
         """Set the transport type protocol for API.
 
         Supports http and https transport types.
         """
-        if not transport_type:
-            raise ValueError('No transport type specified')
         if transport_type.lower() not in (
                 NaServer.TRANSPORT_TYPE_HTTP,
                 NaServer.TRANSPORT_TYPE_HTTPS):
@@ -112,6 +90,10 @@ class NaServer(object):
                 self.set_port(8488)
         self._refresh_conn = True
 
+    def get_style(self):
+        """Get the authorization style for communicating with the server."""
+        return self._auth_style
+
     def set_style(self, style):
         """Set the authorization style for communicating with the server.
 
@@ -122,6 +104,10 @@ class NaServer(object):
             raise ValueError('Unsupported authentication style')
         self._auth_style = style.lower()
 
+    def get_server_type(self):
+        """Get the target server type."""
+        return self._server_type
+
     def set_server_type(self, server_type):
         """Set the target server type.
 
@@ -138,18 +124,13 @@ class NaServer(object):
         self._ns = NaServer.NETAPP_NS
         self._refresh_conn = True
 
-    def set_ontap_version(self, version):
-        self._ontap_version = version
-
-    def get_ontap_version(self):
-        return self._ontap_version
-
     def set_api_version(self, major, minor):
         """Set the API version."""
         try:
             self._api_major_version = int(major)
             self._api_minor_version = int(minor)
-            self._api_version = str(major) + "." + str(minor)
+            self._api_version = six.text_type(major) + "." + \
+                six.text_type(minor)
         except ValueError:
             raise ValueError('Major and minor versions must be integers')
         self._refresh_conn = True
@@ -166,9 +147,13 @@ class NaServer(object):
             int(port)
         except ValueError:
             raise ValueError('Port must be integer')
-        self._port = str(port)
+        self._port = six.text_type(port)
         self._refresh_conn = True
 
+    def get_port(self):
+        """Get the server communication port."""
+        return self._port
+
     def set_timeout(self, seconds):
         """Sets the timeout in seconds."""
         try:
@@ -176,23 +161,43 @@ class NaServer(object):
         except ValueError:
             raise ValueError('timeout in seconds must be integer')
 
+    def get_timeout(self):
+        """Gets the timeout in seconds if set."""
+        if hasattr(self, '_timeout'):
+            return self._timeout
+        return None
+
+    def get_vfiler(self):
+        """Get the vfiler to use in tunneling."""
+        return self._vfiler
+
     def set_vfiler(self, vfiler):
         """Set the vfiler to use if tunneling gets enabled."""
         self._vfiler = vfiler
 
+    def get_vserver(self):
+        """Get the vserver to use in tunneling."""
+        return self._vserver
+
     def set_vserver(self, vserver):
         """Set the vserver to use if tunneling gets enabled."""
         self._vserver = vserver
 
-    @volume_utils.trace_api(filter_function=na_utils.trace_filter_func_api)
-    def send_http_request(self, na_element, enable_tunneling=False):
+    def set_username(self, username):
+        """Set the user name for authentication."""
+        self._username = username
+        self._refresh_conn = True
+
+    def set_password(self, password):
+        """Set the password for authentication."""
+        self._password = password
+        self._refresh_conn = True
+
+    def invoke_elem(self, na_element, enable_tunneling=False):
         """Invoke the API on the server."""
-        if not na_element or not isinstance(na_element, NaElement):
-            raise ValueError('NaElement must be supplied to invoke API')
-
-        request, request_element = self._create_request(na_element,
-                                                        enable_tunneling)
-
+        if na_element and not isinstance(na_element, NaElement):
+            ValueError('NaElement must be supplied to invoke API')
+        request = self._create_request(na_element, enable_tunneling)
         if not hasattr(self, '_opener') or not self._opener \
                 or self._refresh_conn:
             self._build_opener()
@@ -201,16 +206,12 @@ class NaServer(object):
                 response = self._opener.open(request, timeout=self._timeout)
             else:
                 response = self._opener.open(request)
-        except urllib.error.HTTPError as e:
+        except urllib2.HTTPError as e:
             raise NaApiError(e.code, e.msg)
-        except Exception:
-            LOG.exception("Error communicating with NetApp filer.")
-            raise NaApiError('Unexpected error')
-
-        response_xml = response.read()
-        response_element = self._get_result(response_xml)
-
-        return response_element
+        except Exception as e:
+            raise NaApiError('Unexpected error', e)
+        xml = response.read()
+        return self._get_result(xml)
 
     def invoke_successfully(self, na_element, enable_tunneling=False):
         """Invokes API and checks execution status as success.
@@ -220,7 +221,7 @@ class NaServer(object):
         tunneling. The vserver or vfiler should be set before this call
         otherwise tunneling remains disabled.
         """
-        result = self.send_http_request(na_element, enable_tunneling)
+        result = self.invoke_elem(na_element, enable_tunneling)
         if result.has_attr('status') and result.get_attr('status') == 'passed':
             return result
         code = result.get_attr('errno')\
@@ -234,13 +235,6 @@ class NaServer(object):
                 or 'Execution status is failed due to unknown reason'
         raise NaApiError(code, msg)
 
-    def send_request(self, api_name, api_args=None, enable_tunneling=True):
-        """Sends request to Ontapi."""
-        request = NaElement(api_name)
-        if api_args:
-            request.translate_struct(api_args)
-        return self.invoke_successfully(request, enable_tunneling)
-
     def _create_request(self, na_element, enable_tunneling=False):
         """Creates request in the desired format."""
         netapp_elem = NaElement('netapp')
@@ -251,10 +245,10 @@ class NaServer(object):
             self._enable_tunnel_request(netapp_elem)
         netapp_elem.add_child_elem(na_element)
         request_d = netapp_elem.to_string()
-        request = urllib.request.Request(
+        request = urllib2.Request(
             self._get_url(), data=request_d,
             headers={'Content-Type': 'text/xml', 'charset': 'utf-8'})
-        return request, netapp_elem
+        return request
 
     def _enable_tunnel_request(self, netapp_elem):
         """Enables vserver or vfiler tunneling."""
@@ -290,12 +284,7 @@ class NaServer(object):
         return processed_response.get_child_by_name('results')
 
     def _get_url(self):
-        host = self._host
-
-        if netutils.is_valid_ipv6(host):
-            host = netutils.escape_ipv6(host)
-
-        return '%s://%s:%s/%s' % (self._protocol, host, self._port,
+        return '%s://%s:%s/%s' % (self._protocol, self._host, self._port,
                                   self._url)
 
     def _build_opener(self):
@@ -303,21 +292,21 @@ class NaServer(object):
             auth_handler = self._create_basic_auth_handler()
         else:
             auth_handler = self._create_certificate_auth_handler()
-        opener = urllib.request.build_opener(auth_handler)
+        opener = urllib2.build_opener(auth_handler)
         self._opener = opener
 
     def _create_basic_auth_handler(self):
-        password_man = urllib.request.HTTPPasswordMgrWithDefaultRealm()
+        password_man = urllib2.HTTPPasswordMgrWithDefaultRealm()
         password_man.add_password(None, self._get_url(), self._username,
                                   self._password)
-        auth_handler = urllib.request.HTTPBasicAuthHandler(password_man)
+        auth_handler = urllib2.HTTPBasicAuthHandler(password_man)
         return auth_handler
 
     def _create_certificate_auth_handler(self):
         raise NotImplementedError()
 
     def __str__(self):
-        return "server: %s" % self._host
+        return "server: %s" % (self._host)
 
 
 class NaElement(object):
@@ -346,12 +335,17 @@ class NaElement(object):
         """Add the attribute to the element."""
         self._element.set(name, value)
 
+    def add_attrs(self, **attrs):
+        """Add multiple attributes to the element."""
+        for attr in attrs.keys():
+            self._element.set(attr, attrs.get(attr))
+
     def add_child_elem(self, na_element):
         """Add the child element to the element."""
         if isinstance(na_element, NaElement):
             self._element.append(na_element._element)
             return
-        raise Exception(_('Failed to add child element.'))
+        raise
 
     def get_child_by_name(self, name):
         """Get the child element by the tag name."""
@@ -384,10 +378,10 @@ class NaElement(object):
     def get_attr_names(self):
         """Returns the list of attribute names."""
         attributes = self._element.attrib or {}
-        return list(attributes.keys())
+        return attributes.keys()
 
     def add_new_child(self, name, content, convert=False):
-        """Add child with tag name and content.
+        """Add child with tag name and context.
 
            Convert replaces entity refs to chars.
         """
@@ -408,7 +402,7 @@ class NaElement(object):
     def create_node_with_children(node, **children):
         """Creates and returns named node with children."""
         parent = NaElement(node)
-        for child in children:
+        for child in children.keys():
             parent.add_new_child(child, children.get(child, None))
         return parent
 
@@ -422,22 +416,6 @@ class NaElement(object):
         return etree.tostring(self._element, method=method, encoding=encoding,
                               pretty_print=pretty)
 
-    def __str__(self):
-        xml = self.to_string(pretty=True).decode('utf-8')
-        return xml
-
-    def __eq__(self, other):
-        return str(self) == str(other)
-
-    def __ne__(self, other):
-        return not self.__eq__(other)
-
-    def __hash__(self):
-        return hash(str(self))
-
-    def __repr__(self):
-        return str(self)
-
     def __getitem__(self, key):
         """Dict getter method for NaElement.
 
@@ -467,8 +445,8 @@ class NaElement(object):
                     child = NaElement(key)
                     child.add_child_elem(value)
                     self.add_child_elem(child)
-                elif isinstance(value, (str, int, float)):
-                    self.add_new_child(key, str(value))
+                elif isinstance(value, (str, int, float, long)):
+                    self.add_new_child(key, six.text_type(value))
                 elif isinstance(value, (list, tuple, dict)):
                     child = NaElement(key)
                     child.translate_struct(value)
@@ -484,42 +462,26 @@ class NaElement(object):
         """Convert list, tuple, dict to NaElement and appends.
 
            Example usage:
-
            1.
-
-           .. code-block:: xml
-
-                <root>
-                    <elem1>vl1</elem1>
-                    <elem2>vl2</elem2>
-                    <elem3>vl3</elem3>
-                </root>
-
+           <root>
+               <elem1>vl1</elem1>
+               <elem2>vl2</elem2>
+               <elem3>vl3</elem3>
+           </root>
            The above can be achieved by doing
-
-           .. code-block:: python
-
-                root = NaElement('root')
-                root.translate_struct({'elem1': 'vl1', 'elem2': 'vl2',
-                                       'elem3': 'vl3'})
-
+           root = NaElement('root')
+           root.translate_struct({'elem1': 'vl1', 'elem2': 'vl2',
+                                  'elem3': 'vl3'})
            2.
-
-           .. code-block:: xml
-
-                <root>
-                    <elem1>vl1</elem1>
-                    <elem2>vl2</elem2>
-                    <elem1>vl3</elem1>
-                </root>
-
+           <root>
+               <elem1>vl1</elem1>
+               <elem2>vl2</elem2>
+               <elem1>vl3</elem1>
+           </root>
            The above can be achieved by doing
-
-           .. code-block:: python
-
-                root = NaElement('root')
-                root.translate_struct([{'elem1': 'vl1', 'elem2': 'vl2'},
-                                       {'elem1': 'vl3'}])
+           root = NaElement('root')
+           root.translate_struct([{'elem1': 'vl1', 'elem2': 'vl2'},
+                                  {'elem1': 'vl3'}])
         """
         if isinstance(data_struct, (list, tuple)):
             for el in data_struct:
@@ -534,7 +496,7 @@ class NaElement(object):
                     child.translate_struct(data_struct[k])
                 else:
                     if data_struct[k]:
-                        child.set_content(str(data_struct[k]))
+                        child.set_content(six.text_type(data_struct[k]))
                 self.add_child_elem(child)
         else:
             raise ValueError(_('Type cannot be converted into NaElement.'))
@@ -551,320 +513,86 @@ class NaApiError(Exception):
         return 'NetApp API failed. Reason - %s:%s' % (self.code, self.message)
 
 
-class NaRetryableError(NaApiError):
-    def __str__(self, *args, **kwargs):
-        return 'NetApp API failed. Try again. Reason - %s:%s' % (
-            self.code, self.message)
+NaErrors = {'API_NOT_FOUND': NaApiError('13005', 'Unable to find API'),
+            'INSUFFICIENT_PRIVS': NaApiError('13003',
+                                             'Insufficient privileges')}
 
 
-class SSHUtil(object):
-    """Encapsulates connection logic and command execution for SSH client."""
+def invoke_api(na_server, api_name, api_family='cm', query=None,
+               des_result=None, additional_elems=None,
+               is_iter=False, records=0, tag=None,
+               timeout=0, tunnel=None):
+    """Invokes any given API call to a NetApp server.
 
-    MAX_CONCURRENT_SSH_CONNECTIONS = 5
-    RECV_TIMEOUT = 3
-    CONNECTION_KEEP_ALIVE = 600
-    WAIT_ON_STDOUT_TIMEOUT = 3
-
-    def __init__(self, host, username, password, port=22):
-        self.ssh_pool = self._init_ssh_pool(host, port, username, password)
-
-        # Note(cfouts) Number of SSH connections made to the backend need to be
-        # limited. Use of SSHPool allows connections to be cached and reused
-        # instead of creating a new connection each time a command is executed
-        # via SSH.
-        self.ssh_connect_semaphore = semaphore.Semaphore(
-            self.MAX_CONCURRENT_SSH_CONNECTIONS)
-
-    def _init_ssh_pool(self, host, port, username, password):
-        return ssh_utils.SSHPool(host,
-                                 port,
-                                 self.CONNECTION_KEEP_ALIVE,
-                                 username,
-                                 password)
-
-    def execute_command(self, client, command_text, timeout=RECV_TIMEOUT):
-        LOG.debug("execute_command() - Sending command.")
-        stdin, stdout, stderr = client.exec_command(command_text)
-        stdin.close()
-        self._wait_on_stdout(stdout, timeout)
-        output = stdout.read()
-        LOG.debug("Output of length %(size)d received.",
-                  {'size': len(output)})
-        stdout.close()
-        stderr.close()
-        return output
-
-    def execute_command_with_prompt(self,
-                                    client,
-                                    command,
-                                    expected_prompt_text,
-                                    prompt_response,
-                                    timeout=RECV_TIMEOUT):
-        LOG.debug("execute_command_with_prompt() - Sending command.")
-        stdin, stdout, stderr = client.exec_command(command)
-        self._wait_on_stdout(stdout, timeout)
-        response = stdout.channel.recv(999)
-        if expected_prompt_text not in response.strip().decode():
-            msg = _("Unexpected output. Expected [%(expected)s] but "
-                    "received [%(output)s]") % {
-                'expected': expected_prompt_text,
-                'output': response.strip(),
-            }
-            LOG.error(msg)
-            stdin.close()
-            stdout.close()
-            stderr.close()
-            raise exception.VolumeBackendAPIException(msg)
+        :param na_server: na_server instance
+        :param api_name: API name string
+        :param api_family: cm or 7m
+        :param query: API query as dict
+        :param des_result: desired result as dict
+        :param additional_elems: dict other than query and des_result
+        :param is_iter: is iterator API
+        :param records: limit for records, 0 for infinite
+        :param timeout: timeout seconds
+        :param tunnel: tunnel entity, vserver or vfiler name
+    """
+    record_step = 50
+    if not (na_server or isinstance(na_server, NaServer)):
+        msg = _("Requires an NaServer instance.")
+        raise exception.InvalidInput(reason=msg)
+    server = copy.copy(na_server)
+    if api_family == 'cm':
+        server.set_vserver(tunnel)
+    else:
+        server.set_vfiler(tunnel)
+    if timeout > 0:
+        server.set_timeout(timeout)
+    iter_records = 0
+    cond = True
+    while cond:
+        na_element = create_api_request(
+            api_name, query, des_result, additional_elems,
+            is_iter, record_step, tag)
+        result = server.invoke_successfully(na_element, True)
+        if is_iter:
+            if records > 0:
+                iter_records = iter_records + record_step
+                if iter_records >= records:
+                    cond = False
+            tag_el = result.get_child_by_name('next-tag')
+            tag = tag_el.get_content() if tag_el else None
+            if not tag:
+                cond = False
         else:
-            LOG.debug("execute_command_with_prompt() - Sending answer")
-            stdin.write(prompt_response + '\n')
-            stdin.flush()
-        stdin.close()
-        stdout.close()
-        stderr.close()
-
-    def _wait_on_stdout(self, stdout, timeout=WAIT_ON_STDOUT_TIMEOUT):
-        wait_time = 0.0
-        # NOTE(cfouts): The server does not always indicate when EOF is reached
-        # for stdout. The timeout exists for this reason and an attempt is made
-        # to read from stdout.
-        while not stdout.channel.exit_status_ready():
-            # period is 10 - 25 centiseconds
-            period = random.randint(10, 25) / 100.0
-            greenthread.sleep(period)
-            wait_time += period
-            if wait_time > timeout:
-                LOG.debug("Timeout exceeded while waiting for exit status.")
-                break
+            cond = False
+        yield result
 
 
-# REST API error codes.
-REST_UNAUTHORIZED = '6'
-REST_API_NOT_FOUND = '3'
-REST_UPDATE_SNAPMIRROR_FAILED = '13303844'
-REST_ERELATION_EXISTS = '6619637'
-REST_SNAPMIRROR_IN_PROGRESS = '13303810'
-REST_UPDATE_SNAPMIRROR_FAILED = '13303844'
-REST_NO_SUCH_LUN_MAP = '5374922'
-REST_NO_SUCH_FILE = '6684674'
-REST_NAMESPACE_EOBJECTNOTFOUND = ('72090006', '72090006')
+def create_api_request(api_name, query=None, des_result=None,
+                       additional_elems=None, is_iter=False,
+                       record_step=50, tag=None):
+    """Creates a NetApp API request.
 
-
-class RestNaServer(object):
-
-    TRANSPORT_TYPE_HTTP = 'http'
-    TRANSPORT_TYPE_HTTPS = 'https'
-    HTTP_PORT = '80'
-    HTTPS_PORT = '443'
-
-    TRANSPORT_PORT = {
-        TRANSPORT_TYPE_HTTP: HTTP_PORT,
-        TRANSPORT_TYPE_HTTPS: HTTPS_PORT
-    }
-
-    def __init__(self, host, transport_type=TRANSPORT_TYPE_HTTP,
-                 ssl_cert_path=None, username=None, password=None, port=None,
-                 api_trace_pattern=None):
-        self._host = host
-        self.set_transport_type(transport_type)
-        self.set_port(port=port)
-        self._username = username
-        self._password = password
-
-        if api_trace_pattern is not None:
-            na_utils.setup_api_trace_pattern(api_trace_pattern)
-
-        if ssl_cert_path is not None:
-            self._ssl_verify = ssl_cert_path
-        else:
-            # Note(felipe_rodrigues): it will verify with the Mozila CA roots,
-            # given by certifi package.
-            self._ssl_verify = True
-
-        self._api_version = None
-        self._api_major_version = None
-        self._api_minor_version = None
-        self._ontap_version = None
-        self._timeout = None
-
-        LOG.debug('Using REST with NetApp controller: %s', self._host)
-
-    def set_transport_type(self, transport_type):
-        """Set the transport type protocol for API.
-
-        Supports http and https transport types.
-        """
-        if transport_type is None or transport_type.lower() not in (
-                RestNaServer.TRANSPORT_TYPE_HTTP,
-                RestNaServer.TRANSPORT_TYPE_HTTPS):
-            raise ValueError('Unsupported transport type')
-        self._protocol = transport_type.lower()
-
-    def get_transport_type(self):
-        """Get the transport type protocol."""
-        return self._protocol
-
-    def set_api_version(self, major, minor):
-        """Set the API version."""
-        try:
-            self._api_major_version = int(major)
-            self._api_minor_version = int(minor)
-            self._api_version = str(major) + "." + str(minor)
-        except ValueError:
-            raise ValueError('Major and minor versions must be integers')
-
-    def get_api_version(self):
-        """Gets the API version tuple."""
-        if not self._api_version:
-            return None
-        return (self._api_major_version, self._api_minor_version)
-
-    def set_ontap_version(self, ontap_version):
-        """Set the ONTAP version."""
-        self._ontap_version = ontap_version
-
-    def get_ontap_version(self):
-        """Gets the ONTAP version."""
-        return self._ontap_version
-
-    def set_port(self, port=None):
-        """Set the ONTAP port, if not informed, set with default one."""
-        if port is None and self._protocol in RestNaServer.TRANSPORT_PORT:
-            self._port = RestNaServer.TRANSPORT_PORT[self._protocol]
-        else:
-            try:
-                int(port)
-            except ValueError:
-                raise ValueError('Port must be integer')
-            self._port = str(port)
-
-    def get_port(self):
-        """Get the server communication port."""
-        return self._port
-
-    def set_timeout(self, seconds):
-        """Sets the timeout in seconds."""
-        try:
-            self._timeout = int(seconds)
-        except ValueError:
-            raise ValueError('timeout in seconds must be integer')
-
-    def get_timeout(self):
-        """Gets the timeout in seconds if set."""
-        return self._timeout
-
-    def set_vserver(self, vserver):
-        """Set the vserver to use if tunneling gets enabled."""
-        self._vserver = vserver
-
-    def get_vserver(self):
-        """Get the vserver to use in tunneling."""
-        return self._vserver
-
-    def __str__(self):
-        """Gets a representation of the client."""
-        return "server: %s" % (self._host)
-
-    def _get_request_method(self, method, session):
-        """Returns the request method to be used in the REST call."""
-
-        request_methods = {
-            'post': session.post,
-            'get': session.get,
-            'put': session.put,
-            'delete': session.delete,
-            'patch': session.patch,
-        }
-        return request_methods[method]
-
-    def _add_query_params_to_url(self, url, query):
-        """Populates the URL with specified filters."""
-        filters = '&'.join([f"{k}={v}" for k, v in query.items()])
-        url += "?" + filters
-        return url
-
-    def _get_base_url(self):
-        """Get the base URL for REST requests."""
-        host = self._host
-        if ':' in host:
-            host = '[%s]' % host
-        return '%s://%s:%s/api/' % (self._protocol, host, self._port)
-
-    def _build_session(self, headers):
-        """Builds a session in the client."""
-        self._session = requests.Session()
-
-        # NOTE(felipe_rodrigues): request resilient of temporary network
-        # failures (like name resolution failure), retrying until 5 times.
-        max_retries = Retry(total=5, connect=5, read=2, backoff_factor=1)
-        adapter = HTTPAdapter(max_retries=max_retries)
-        self._session.mount('%s://' % self._protocol, adapter)
-
-        self._session.auth = self._create_basic_auth_handler()
-        self._session.verify = self._ssl_verify
-        self._session.headers = headers
-
-    def _build_headers(self, enable_tunneling):
-        """Build and return headers for a REST request."""
-        headers = {
-            "Accept": "application/json",
-            "Content-Type": "application/json"
-        }
-        if enable_tunneling:
-            headers["X-Dot-SVM-Name"] = self.get_vserver()
-
-        return headers
-
-    def _create_basic_auth_handler(self):
-        """Creates and returns a basic HTTP auth handler."""
-        return auth.HTTPBasicAuth(self._username, self._password)
-
-    @volume_utils.trace_api(
-        filter_function=na_utils.trace_filter_func_rest_api)
-    def send_http_request(self, method, url, body, headers):
-        """Invoke the API on the server.
-
-        The passed parameters and returned parameters will be logged if trace
-        feature is on. They are important for debugging purpose.
-        """
-        data = jsonutils.dumps(body) if body else {}
-
-        self._build_session(headers)
-        request_method = self._get_request_method(method, self._session)
-
-        try:
-            if self._timeout is not None:
-                response = request_method(
-                    url, data=data, timeout=self._timeout)
-            else:
-                response = request_method(url, data=data)
-        except requests.HTTPError as e:
-            raise NaApiError(e.errno, e.strerror)
-        except Exception as e:
-            raise NaApiError(message=e)
-
-        code = response.status_code
-        body = jsonutils.loads(response.content) if response.content else {}
-        return code, body
-
-    def invoke_successfully(self, action_url, method, body=None, query=None,
-                            enable_tunneling=False):
-        """Invokes REST API and checks execution status as success."""
-        headers = self._build_headers(enable_tunneling)
-        if query:
-            action_url = self._add_query_params_to_url(action_url, query)
-        url = self._get_base_url() + action_url
-        code, response = self.send_http_request(method, url, body, headers)
-
-        if not response.get('error'):
-            return code, response
-
-        result_error = response.get('error')
-        code = result_error.get('code', 'ESTATUSFAILED')
-        # TODO: add the correct code number for REST not licensed clone error.
-        if code == ESIS_CLONE_NOT_LICENSED:
-            msg = 'Clone operation failed: FlexClone not licensed.'
-        else:
-            msg = (result_error.get('message')
-                   or 'Execution status is failed due to unknown reason')
-        raise NaApiError(code, msg)
+        :param api_name: API name string
+        :param query: API query as dict
+        :param des_result: desired result as dict
+        :param additional_elems: dict other than query and des_result
+        :param is_iter: is iterator API
+        :param record_step: records at a time for iter API
+        :param tag: next tag for iter API
+    """
+    api_el = NaElement(api_name)
+    if query:
+        query_el = NaElement('query')
+        query_el.translate_struct(query)
+        api_el.add_child_elem(query_el)
+    if des_result:
+        res_el = NaElement('desired-attributes')
+        res_el.translate_struct(des_result)
+        api_el.add_child_elem(res_el)
+    if additional_elems:
+        api_el.translate_struct(additional_elems)
+    if is_iter:
+        api_el.add_new_child('max-records', six.text_type(record_step))
+    if tag:
+        api_el.add_new_child('tag', tag, True)
+    return api_el
