@@ -4,7 +4,6 @@
 # Copyright (c) 2014 Clinton Knight.  All rights reserved.
 # Copyright (c) 2014 Alex Meade.  All rights reserved.
 # Copyright (c) 2014 Bob Callaway.  All rights reserved.
-# Copyright (c) 2015 Tom Barron.  All rights reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -21,18 +20,14 @@
 Volume driver for NetApp NFS storage.
 """
 
-import os
-
 from oslo_log import log as logging
+from oslo_utils import units
 import six
 
 from cinder import exception
-from cinder.i18n import _
-from cinder import interface
-from cinder import utils
+from cinder.i18n import _, _LE, _LI
 from cinder.volume.drivers.netapp.dataontap.client import client_7mode
 from cinder.volume.drivers.netapp.dataontap import nfs_base
-from cinder.volume.drivers.netapp.dataontap.performance import perf_7mode
 from cinder.volume.drivers.netapp import options as na_opts
 from cinder.volume.drivers.netapp import utils as na_utils
 from cinder.volume import utils as volume_utils
@@ -41,14 +36,9 @@ from cinder.volume import utils as volume_utils
 LOG = logging.getLogger(__name__)
 
 
-@six.add_metaclass(utils.TraceWrapperWithABCMetaclass)
-@interface.volumedriver
 class NetApp7modeNfsDriver(nfs_base.NetAppNfsDriver):
     """NetApp NFS driver for Data ONTAP (7-mode)."""
 
-    # ThirdPartySystems wiki page
-    CI_WIKI_NAME = "NetApp_CI"
-
     def __init__(self, *args, **kwargs):
         super(NetApp7modeNfsDriver, self).__init__(*args, **kwargs)
         self.configuration.append_config_values(na_opts.netapp_7mode_opts)
@@ -65,9 +55,6 @@ class NetApp7modeNfsDriver(nfs_base.NetAppNfsDriver):
             port=self.configuration.netapp_server_port,
             vfiler=self.configuration.netapp_vfiler)
 
-        self.perf_library = perf_7mode.Performance7modeLibrary(
-            self.zapi_client)
-
     def check_for_setup_error(self):
         """Checks if setup occurred properly."""
         api_version = self.zapi_client.get_ontapi_version()
@@ -80,26 +67,49 @@ class NetApp7modeNfsDriver(nfs_base.NetAppNfsDriver):
         else:
             msg = _("Data ONTAP API version could not be determined.")
             raise exception.VolumeBackendAPIException(data=msg)
-        self._add_looping_tasks()
         super(NetApp7modeNfsDriver, self).check_for_setup_error()
 
-    def _add_looping_tasks(self):
-        """Add tasks that need to be executed at a fixed interval."""
-        super(NetApp7modeNfsDriver, self)._add_looping_tasks()
+    def create_volume(self, volume):
+        """Creates a volume.
 
-    def _clone_backing_file_for_volume(self, volume_name, clone_name,
-                                       volume_id, share=None,
-                                       is_snapshot=False,
-                                       source_snapshot=None):
-        """Clone backing file for Cinder volume.
-
-        :param: is_snapshot Not used, present for method signature consistency
+        :param volume: volume reference
         """
+        LOG.debug('create_volume on %s' % volume['host'])
+        self._ensure_shares_mounted()
+
+        # get share as pool name
+        share = volume_utils.extract_host(volume['host'], level='pool')
+
+        if share is None:
+            msg = _("Pool is not available in the volume host field.")
+            raise exception.InvalidHost(reason=msg)
+
+        volume['provider_location'] = share
+        LOG.info(_LI('Creating volume at location %s')
+                 % volume['provider_location'])
+
+        try:
+            self._do_create_volume(volume)
+        except Exception as ex:
+            LOG.error(_LE("Exception creating vol %(name)s on "
+                          "share %(share)s. Details: %(ex)s")
+                      % {'name': volume['name'],
+                         'share': volume['provider_location'],
+                         'ex': six.text_type(ex)})
+            msg = _("Volume %s could not be created on shares.")
+            raise exception.VolumeBackendAPIException(
+                data=msg % (volume['name']))
+
+        return {'provider_location': volume['provider_location']}
+
+    def _clone_volume(self, volume_name, clone_name,
+                      volume_id, share=None):
+        """Clones mounted volume with NetApp filer."""
         (_host_ip, export_path) = self._get_export_ip_path(volume_id, share)
         storage_path = self.zapi_client.get_actual_path_for_export(export_path)
         target_path = '%s/%s' % (storage_path, clone_name)
         self.zapi_client.clone_file('%s/%s' % (storage_path, volume_name),
-                                    target_path, source_snapshot)
+                                    target_path)
 
     def _update_volume_stats(self):
         """Retrieve stats info from vserver."""
@@ -114,41 +124,42 @@ class NetApp7modeNfsDriver(nfs_base.NetAppNfsDriver):
         data['vendor_name'] = 'NetApp'
         data['driver_version'] = self.VERSION
         data['storage_protocol'] = 'nfs'
-        data['pools'] = self._get_pool_stats(
-            filter_function=self.get_filter_function(),
-            goodness_function=self.get_goodness_function())
-        data['sparse_copy_volume'] = True
+        data['pools'] = self._get_pool_stats()
 
         self._spawn_clean_cache_job()
         self.zapi_client.provide_ems(self, netapp_backend, self._app_version,
                                      server_type="7mode")
         self._stats = data
 
-    def _get_pool_stats(self, filter_function=None, goodness_function=None):
+    def _get_pool_stats(self):
         """Retrieve pool (i.e. NFS share) stats info from SSC volumes."""
 
         pools = []
-        self.perf_library.update_performance_cache()
 
         for nfs_share in self._mounted_shares:
 
-            capacity = self._get_share_capacity_info(nfs_share)
+            capacity = self._get_extended_capacity_info(nfs_share)
 
             pool = dict()
             pool['pool_name'] = nfs_share
             pool['QoS_support'] = False
-            pool['multiattach'] = True
-            pool.update(capacity)
+            pool['reserved_percentage'] = 0
 
-            thick = not self.configuration.nfs_sparsed_volumes
-            pool['thick_provisioning_support'] = thick
-            pool['thin_provisioning_support'] = not thick
+            # Report pool as reserved when over the configured used_ratio
+            if capacity['used_ratio'] > self.configuration.nfs_used_ratio:
+                pool['reserved_percentage'] = 100
 
-            utilization = self.perf_library.get_node_utilization()
-            pool['utilization'] = na_utils.round_down(utilization, '0.01')
-            pool['filter_function'] = filter_function
-            pool['goodness_function'] = goodness_function
-            pool['consistencygroup_support'] = True
+            # Report pool as reserved when over the subscribed ratio
+            if capacity['subscribed_ratio'] >=\
+                    self.configuration.nfs_oversub_ratio:
+                pool['reserved_percentage'] = 100
+
+            # convert sizes to GB
+            total = float(capacity['apparent_size']) / units.Gi
+            pool['total_capacity_gb'] = na_utils.round_down(total, '0.01')
+
+            free = float(capacity['apparent_available']) / units.Gi
+            pool['free_capacity_gb'] = na_utils.round_down(free, '0.01')
 
             pools.append(pool)
 
@@ -157,13 +168,11 @@ class NetApp7modeNfsDriver(nfs_base.NetAppNfsDriver):
     def _shortlist_del_eligible_files(self, share, old_files):
         """Prepares list of eligible files to be deleted from cache."""
         file_list = []
-        (_, export_path) = self._get_export_ip_path(share=share)
-        exported_volume = self.zapi_client.get_actual_path_for_export(
-            export_path)
-        for old_file in old_files:
-            path = os.path.join(exported_volume, old_file)
+        exp_volume = self.zapi_client.get_actual_path_for_export(share)
+        for file in old_files:
+            path = '/vol/%s/%s' % (exp_volume, file)
             u_bytes = self.zapi_client.get_file_usage(path)
-            file_list.append((old_file, u_bytes))
+            file_list.append((file, u_bytes))
         LOG.debug('Shortlisted files eligible for deletion: %s', file_list)
         return file_list
 
@@ -204,58 +213,16 @@ class NetApp7modeNfsDriver(nfs_base.NetAppNfsDriver):
         LOG.debug('No share match found for ip %s', ip)
         return None
 
-    def _is_share_clone_compatible(self, volume, share):
-        """Checks if share is compatible with volume to host its clone."""
-        thin = self.configuration.nfs_sparsed_volumes
-        return self._share_has_space_for_clone(share, volume['size'], thin)
+    def _is_share_vol_compatible(self, volume, share):
+        """Checks if share is compatible with volume to host it."""
+        return self._is_share_eligible(share, volume['size'])
 
-    def _check_volume_type(self, volume, share, file_name, extra_specs):
+    def _check_volume_type(self, volume, share, file_name):
         """Matches a volume type for share file."""
+        extra_specs = na_utils.get_volume_extra_specs(volume)
         qos_policy_group = extra_specs.pop('netapp:qos_policy_group', None) \
             if extra_specs else None
         if qos_policy_group:
             raise exception.ManageExistingVolumeTypeMismatch(
                 reason=(_("Setting file qos policy group is not supported"
                           " on this storage family and ontap version.")))
-        volume_type = na_utils.get_volume_type_from_volume(volume)
-        if volume_type and 'qos_spec_id' in volume_type:
-            raise exception.ManageExistingVolumeTypeMismatch(
-                reason=_("QoS specs are not supported"
-                         " on this storage family and ONTAP version."))
-
-    def _do_qos_for_volume(self, volume, extra_specs, cleanup=False):
-        """Set QoS policy on backend from volume type information."""
-        # 7-mode DOT does not support QoS.
-        return
-
-    def _get_backing_flexvol_names(self):
-        """Returns a list of backing flexvol names."""
-        flexvol_names = []
-        for nfs_share in self._mounted_shares:
-            flexvol_name = nfs_share.rsplit('/', 1)[1]
-            flexvol_names.append(flexvol_name)
-            LOG.debug("Found flexvol %s", flexvol_name)
-
-        return flexvol_names
-
-    def _get_flexvol_names_from_hosts(self, hosts):
-        """Returns a set of flexvol names."""
-        flexvols = set()
-        for host in hosts:
-            pool_name = volume_utils.extract_host(host, level='pool')
-            flexvol_name = pool_name.rsplit('/', 1)[1]
-            flexvols.add(flexvol_name)
-        return flexvols
-
-    @utils.trace_method
-    def delete_cgsnapshot(self, context, cgsnapshot, snapshots):
-        """Delete files backing each snapshot in the cgsnapshot.
-
-        :return: An implicit update of snapshot models that the manager will
-                 interpret and subsequently set the model state to deleted.
-        """
-        for snapshot in snapshots:
-            self._delete_file(snapshot['volume_id'], snapshot['name'])
-            LOG.debug("Snapshot %s deletion successful", snapshot['name'])
-
-        return None, None
