@@ -1,7 +1,7 @@
-#    (c) Copyright 2014 Brocade Communications Systems Inc.
+#    (c) Copyright 2015 Brocade Communications Systems Inc.
 #    All Rights Reserved.
 #
-#    Copyright 2014 OpenStack Foundation
+#    Copyright 2015 OpenStack Foundation
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -30,15 +30,16 @@ add_connection and delete_connection interfaces.
 """
 
 
-from oslo.config import cfg
+from oslo_concurrency import lockutils
+from oslo_config import cfg
+from oslo_log import log as logging
+from oslo_utils import excutils
+from oslo_utils import importutils
 
 from cinder import exception
-from cinder.openstack.common import excutils
-from cinder.openstack.common import importutils
-from cinder.openstack.common import lockutils
-from cinder.openstack.common import log as logging
+from cinder.i18n import _, _LE, _LI
 from cinder.zonemanager.drivers.brocade import brcd_fabric_opts as fabric_opts
-from cinder.zonemanager.drivers.fc_zone_driver import FCZoneDriver
+from cinder.zonemanager.drivers import fc_zone_driver
 
 LOG = logging.getLogger(__name__)
 
@@ -53,7 +54,7 @@ CONF = cfg.CONF
 CONF.register_opts(brcd_opts, 'fc-zone-manager')
 
 
-class BrcdFCZoneDriver(FCZoneDriver):
+class BrcdFCZoneDriver(fc_zone_driver.FCZoneDriver):
     """Brocade FC zone driver implementation.
 
     OpenStack Fibre Channel zone driver to manage FC zoning in
@@ -75,8 +76,6 @@ class BrcdFCZoneDriver(FCZoneDriver):
             # Adding a hack to hendle parameters from super classes
             # in case configured with multi backend.
             fabric_names = self.configuration.safe_get('fc_fabric_names')
-            activate = self.configuration.safe_get('zone_activate')
-            prefix = self.configuration.safe_get('zone_name_prefix')
             base_san_opts = []
             if not fabric_names:
                 base_san_opts.append(
@@ -86,21 +85,12 @@ class BrcdFCZoneDriver(FCZoneDriver):
                                ' retrieve other SAN credentials for connecting'
                                ' to each SAN fabric'
                                ))
-            if not activate:
-                base_san_opts.append(
-                    cfg.BoolOpt('zone_activate',
-                                default=True,
-                                help='Indicates whether zone should '
-                                'be activated or not'))
-            if not prefix:
-                base_san_opts.append(
-                    cfg.StrOpt('zone_name_prefix',
-                               default="openstack",
-                               help="A prefix to be used when naming zone"))
             if len(base_san_opts) > 0:
                 CONF.register_opts(base_san_opts)
                 self.configuration.append_config_values(base_san_opts)
-            fabric_names = self.configuration.fc_fabric_names.split(',')
+
+            fc_fabric_names = self.configuration.fc_fabric_names
+            fabric_names = [x.strip() for x in fc_fabric_names.split(',')]
 
             # There can be more than one SAN in the network and we need to
             # get credentials for each SAN.
@@ -130,16 +120,22 @@ class BrcdFCZoneDriver(FCZoneDriver):
         :param fabric: Fabric name from cinder.conf file
         :param initiator_target_map: Mapping of initiator to list of targets
         """
-        LOG.debug(_("Add connection for Fabric:%s"), fabric)
-        LOG.info(_("BrcdFCZoneDriver - Add connection "
-                   "for I-T map: %s"), initiator_target_map)
+        LOG.debug("Add connection for Fabric:%s", fabric)
+        LOG.info(_LI("BrcdFCZoneDriver - Add connection "
+                     "for I-T map: %s"), initiator_target_map)
         zoning_policy = self.configuration.zoning_policy
         zoning_policy_fab = self.fabric_configs[fabric].safe_get(
             'zoning_policy')
         if zoning_policy_fab:
             zoning_policy = zoning_policy_fab
+        zone_name_prefix = self.fabric_configs[fabric].safe_get(
+            'zone_name_prefix')
+        if not zone_name_prefix:
+            zone_name_prefix = 'openstack'
+        zone_activate = self.fabric_configs[fabric].safe_get(
+            'zone_activate')
 
-        LOG.info(_("Zoning policy for Fabric %s"), zoning_policy)
+        LOG.info(_LI("Zoning policy for Fabric %s"), zoning_policy)
         cli_client = self._get_cli_client(fabric)
         cfgmap_from_fabric = self._get_active_zone_set(cli_client)
 
@@ -157,7 +153,7 @@ class BrcdFCZoneDriver(FCZoneDriver):
                     target = t.lower()
                     zone_members = [self.get_formatted_wwn(initiator),
                                     self.get_formatted_wwn(target)]
-                    zone_name = (self.configuration.zone_name_prefix
+                    zone_name = (zone_name_prefix
                                  + initiator.replace(':', '')
                                  + target.replace(':', ''))
                     if (
@@ -166,16 +162,15 @@ class BrcdFCZoneDriver(FCZoneDriver):
                         zone_map[zone_name] = zone_members
                     else:
                         # This is I-T zoning, skip if zone already exists.
-                        LOG.info(_("Zone exists in I-T mode. "
-                                   "Skipping zone creation %s"), zone_name)
+                        LOG.info(_LI("Zone exists in I-T mode. "
+                                     "Skipping zone creation %s"), zone_name)
             elif zoning_policy == 'initiator':
                 zone_members = [self.get_formatted_wwn(initiator)]
                 for t in t_list:
                     target = t.lower()
                     zone_members.append(self.get_formatted_wwn(target))
 
-                zone_name = self.configuration.zone_name_prefix \
-                    + initiator.replace(':', '')
+                zone_name = zone_name_prefix + initiator.replace(':', '')
 
                 if len(zone_names) > 0 and (zone_name in zone_names):
                     zone_members = zone_members + filter(
@@ -189,12 +184,12 @@ class BrcdFCZoneDriver(FCZoneDriver):
                 LOG.error(msg)
                 raise exception.FCZoneDriverException(msg)
 
-            LOG.info(_("Zone map to add: %s"), zone_map)
+            LOG.info(_LI("Zone map to add: %s"), zone_map)
 
             if len(zone_map) > 0:
                 try:
                     cli_client.add_zones(
-                        zone_map, self.configuration.zone_activate,
+                        zone_map, zone_activate,
                         cfgmap_from_fabric)
                     cli_client.cleanup()
                 except exception.BrocadeZoningCliException as brocade_ex:
@@ -203,7 +198,7 @@ class BrcdFCZoneDriver(FCZoneDriver):
                     LOG.error(e)
                     msg = _("Failed to add zoning configuration %s") % e
                     raise exception.FCZoneDriverException(msg)
-            LOG.debug(_("Zones added successfully: %s"), zone_map)
+            LOG.debug("Zones added successfully: %s", zone_map)
 
     @lockutils.synchronized('brcd', 'fcfabric-', True)
     def delete_connection(self, fabric, initiator_target_map):
@@ -216,16 +211,22 @@ class BrcdFCZoneDriver(FCZoneDriver):
         :param fabric: Fabric name from cinder.conf file
         :param initiator_target_map: Mapping of initiator to list of targets
         """
-        LOG.debug(_("Delete connection for fabric:%s"), fabric)
-        LOG.info(_("BrcdFCZoneDriver - Delete connection for I-T map: %s"),
+        LOG.debug("Delete connection for fabric:%s", fabric)
+        LOG.info(_LI("BrcdFCZoneDriver - Delete connection for I-T map: %s"),
                  initiator_target_map)
         zoning_policy = self.configuration.zoning_policy
         zoning_policy_fab = self.fabric_configs[fabric].safe_get(
             'zoning_policy')
         if zoning_policy_fab:
             zoning_policy = zoning_policy_fab
+        zone_name_prefix = self.fabric_configs[fabric].safe_get(
+            'zone_name_prefix')
+        if not zone_name_prefix:
+            zone_name_prefix = 'openstack'
+        zone_activate = self.fabric_configs[fabric].safe_get(
+            'zone_activate')
 
-        LOG.info(_("Zoning policy for fabric %s"), zoning_policy)
+        LOG.info(_LI("Zoning policy for fabric %s"), zoning_policy)
         conn = self._get_cli_client(fabric)
         cfgmap_from_fabric = self._get_active_zone_set(conn)
 
@@ -236,7 +237,7 @@ class BrcdFCZoneDriver(FCZoneDriver):
         # Based on zoning policy, get zone member list and push changes to
         # fabric. This operation could result in an update for zone config
         # with new member list or deleting zones from active cfg.
-        LOG.debug(_("zone config from Fabric: %s"), cfgmap_from_fabric)
+        LOG.debug("zone config from Fabric: %s", cfgmap_from_fabric)
         for initiator_key in initiator_target_map.keys():
             initiator = initiator_key.lower()
             formatted_initiator = self.get_formatted_wwn(initiator)
@@ -248,10 +249,10 @@ class BrcdFCZoneDriver(FCZoneDriver):
                 for t in t_list:
                     target = t.lower()
                     zone_name = (
-                        self.configuration.zone_name_prefix
+                        zone_name_prefix
                         + initiator.replace(':', '')
                         + target.replace(':', ''))
-                    LOG.debug(_("Zone name to del: %s"), zone_name)
+                    LOG.debug("Zone name to del: %s", zone_name)
                     if len(zone_names) > 0 and (zone_name in zone_names):
                         # delete zone.
                         LOG.debug("Added zone to delete to "
@@ -264,8 +265,7 @@ class BrcdFCZoneDriver(FCZoneDriver):
                     target = t.lower()
                     zone_members.append(self.get_formatted_wwn(target))
 
-                zone_name = self.configuration.zone_name_prefix \
-                    + initiator.replace(':', '')
+                zone_name = zone_name_prefix + initiator.replace(':', '')
 
                 if (zone_names and (zone_name in zone_names)):
                     filtered_members = filter(
@@ -277,27 +277,27 @@ class BrcdFCZoneDriver(FCZoneDriver):
                     # filtered list and if it is non-empty, add initiator
                     # to it and update zone if filtered list is empty, we
                     # remove that zone.
-                    LOG.debug(_("Zone delete - I mode: "
-                                "filtered targets:%s"), filtered_members)
+                    LOG.debug("Zone delete - I mode: "
+                              "filtered targets:%s", filtered_members)
                     if filtered_members:
                         filtered_members.append(formatted_initiator)
-                        LOG.debug(_("Filtered zone members to "
-                                    "update: %s"), filtered_members)
+                        LOG.debug("Filtered zone members to "
+                                  "update: %s", filtered_members)
                         zone_map[zone_name] = filtered_members
-                        LOG.debug(_("Filtered zone Map to "
-                                    "update: %s"), zone_map)
+                        LOG.debug("Filtered zone Map to "
+                                  "update: %s", zone_map)
                     else:
                         zones_to_delete.append(zone_name)
             else:
-                LOG.info(_("Zoning Policy: %s, not "
-                           "recognized"), zoning_policy)
-            LOG.debug(_("Final Zone map to update: %s"), zone_map)
-            LOG.debug(_("Final Zone list to delete: %s"), zones_to_delete)
+                LOG.info(_LI("Zoning Policy: %s, not "
+                             "recognized"), zoning_policy)
+            LOG.debug("Final Zone map to update: %s", zone_map)
+            LOG.debug("Final Zone list to delete: %s", zones_to_delete)
             try:
                 # Update zone membership.
                 if zone_map:
                     conn.add_zones(
-                        zone_map, self.configuration.zone_activate,
+                        zone_map, zone_activate,
                         cfgmap_from_fabric)
                 # Delete zones ~sk.
                 if zones_to_delete:
@@ -313,7 +313,7 @@ class BrcdFCZoneDriver(FCZoneDriver):
                                 zone_name_string, ';', zones_to_delete[i])
 
                     conn.delete_zones(
-                        zone_name_string, self.configuration.zone_activate,
+                        zone_name_string, zone_activate,
                         cfgmap_from_fabric)
                 conn.cleanup()
             except Exception as e:
@@ -330,14 +330,15 @@ class BrcdFCZoneDriver(FCZoneDriver):
         # TODO(Santhosh Kolathur): consider refactoring to use lookup service.
         formatted_target_list = []
         fabric_map = {}
-        fabrics = self.configuration.fc_fabric_names.split(',')
-        LOG.debug(_("Fabric List: %s"), fabrics)
-        LOG.debug(_("Target wwn List: %s"), target_wwn_list)
+        fc_fabric_names = self.configuration.fc_fabric_names
+        fabrics = [x.strip() for x in fc_fabric_names.split(',')]
+        LOG.debug("Fabric List: %s", fabrics)
+        LOG.debug("Target wwn List: %s", target_wwn_list)
         if len(fabrics) > 0:
             for t in target_wwn_list:
                 formatted_target_list.append(self.get_formatted_wwn(t.lower()))
-            LOG.debug(_("Formatted Target wwn List:"
-                        " %s"), formatted_target_list)
+            LOG.debug("Formatted Target wwn List:"
+                      " %s", formatted_target_list)
             for fabric_name in fabrics:
                 conn = self._get_cli_client(fabric_name)
 
@@ -346,7 +347,7 @@ class BrcdFCZoneDriver(FCZoneDriver):
                 nsinfo = None
                 try:
                     nsinfo = conn.get_nameserver_info()
-                    LOG.debug(_("name server info from fabric:%s"), nsinfo)
+                    LOG.debug("name server info from fabric:%s", nsinfo)
                     conn.cleanup()
                 except exception.BrocadeZoningCliException as ex:
                     if not conn.is_supported_firmware():
@@ -356,8 +357,8 @@ class BrcdFCZoneDriver(FCZoneDriver):
                         LOG.error(msg)
                         raise exception.FCZoneDriverException(msg)
                     with excutils.save_and_reraise_exception():
-                        LOG.error(_("Error getting name server "
-                                    "info: %s"), ex)
+                        LOG.error(_LE("Error getting name server "
+                                      "info: %s"), ex)
                 except Exception as e:
                     msg = (_("Failed to get name server info:%s") % e)
                     LOG.error(msg)
@@ -367,7 +368,7 @@ class BrcdFCZoneDriver(FCZoneDriver):
                     nsinfo)
 
                 if visible_targets:
-                    LOG.info(_("Filtered targets for SAN is: %s"),
+                    LOG.info(_LI("Filtered targets for SAN is: %s"),
                              {fabric_name: visible_targets})
                     # getting rid of the ':' before returning
                     for idx, elem in enumerate(visible_targets):
@@ -375,9 +376,9 @@ class BrcdFCZoneDriver(FCZoneDriver):
                             visible_targets[idx]).replace(':', '')
                     fabric_map[fabric_name] = visible_targets
                 else:
-                    LOG.debug(_("No targets are in the nameserver for SAN %s"),
+                    LOG.debug("No targets are in the nameserver for SAN %s",
                               fabric_name)
-        LOG.debug(_("Return SAN context output:%s"), fabric_map)
+        LOG.debug("Return SAN context output:%s", fabric_map)
         return fabric_map
 
     def _get_active_zone_set(self, conn):
@@ -395,7 +396,7 @@ class BrcdFCZoneDriver(FCZoneDriver):
             LOG.error(e)
             msg = _("Failed to retrieve active zoning configuration %s") % e
             raise exception.FCZoneDriverException(msg)
-        LOG.debug(_("Active zone set from fabric: %s"), cfgmap)
+        LOG.debug("Active zone set from fabric: %s", cfgmap)
         return cfgmap
 
     def _get_cli_client(self, fabric):
